import 'dart:collection';

import 'package:meta/meta.dart';

/// `DelegateListMixin` can be used as a mixin to implement a delegating list. It
/// delegates all methods to [delegate].
mixin DelegateListMixin<T> on ListMixin<T> {

  @override
  T operator[](int index) => delegate[index];

  @override
  void operator[]=(int index, T value) => delegate[index] = value;

  @override
  int get length => delegate.length;

  @override
  set length(int length) => delegate.length = length;


  /// The underlying list to which all methods are delegated.
  @visibleForOverriding
  @protected List<T> get delegate;

}


/// A list that delegates all methods to an underlying list.
class DelegateList<T> with ListMixin<T>, DelegateListMixin<T> {

  final List<T> _list;

  /// Creates a `DelegateList` that delegates all methods to the given [_list].
  DelegateList(this._list);

  @override
  @visibleForTesting
  @protected List<T> get delegate => _list;

}