import 'dart:collection';

import 'package:meta/meta.dart';


/// `DelegateSetMixin` can be used as a mixin to implement a delegating set. It
/// delegates all methods to [delegate].
mixin DelegateSetMixin<T> on SetMixin<T> {

  @override
  bool add(T value) => delegate.add(value);

  @override
  bool contains(Object element) => delegate.contains(element);

  @override
  T lookup(Object element) => delegate.lookup(element);

  @override
  bool remove(Object value) => delegate.remove(value);

  @override
  Iterator<T> get iterator => delegate.iterator;

  @override
  int get length => delegate.length;

  /// The underlying set to which all calls are delegated.
  @visibleForOverriding
  @protected Set<T> get delegate;

}


/// A set that delegates all methods to an underlying set.
class DelegateSet<T> with SetMixin<T>, DelegateSetMixin<T> {

  final Set<T> _set;

  /// Creates a `DelegateSet` that delegates all methods to the given [_set].
  DelegateSet(this._set);


  @override
  @visibleForTesting
  @protected Set<T> get delegate => _set;


  /// Copies this set and forward all methods to a copy of [_set].
  @override
  DelegateSet<T> toSet() => DelegateSet(_set.toSet());

}
