import 'package:sane/core.dart';

import 'delegate_list.dart';
import 'observable_modification.dart';


/// A list that emits `Modification.add` and `Modification.clear` events when it is modified.
/// The key of a `Modification.add` event is an index. All methods are delegated to an
/// underlying list.
class ObservableList<T> extends DelegateList<T> with Listenable<Modification<int, T>> {

  /// Creates an `ObservableList` that delegates all methods to [list].
  /// If [list] is not given, an empty list is used.
  ObservableList([List<T> list]): super(list ?? []);


  /// Sets the value at the given index in the list to value or throws a [RangeError]
  /// if index is out of bounds; always emits a `Modification.add` event.
  @override
  void operator[]=(int index, T value) {
    delegate[index] = value;
    source.add(Modification.add(index, value));
  }

  /// Removes all objects from this list; the length of the list becomes zero;
  /// always emits a `Modification.clear` event.
  @override
  void clear() {
    delegate.clear();
    source.add(const Modification.clear());
  }

}