import 'package:meta/meta.dart';

import 'package:sane/core.dart';

/// The possible kinds of modification.
enum Kind {

  /// Denotes an addition.
  add,
  /// Denotes a removal.
  remove,
  /// Denotes that the collection has been cleared.
  clear

}


/// A `Modification` represents the modification of a collection.
@immutable class Modification<K, V> {

  /// The modification kind.
  final Kind kind;
  /// A `key` of a modified entry.
  @nullable final K key;
  /// A `value` of a modified entry.
  @nullable final V value;

  /// Creates a `Modification` that denotes an addition with the given [key] and [value].
  @literal const Modification.add(this.key, this.value): kind = Kind.add;

  /// Creates a `Modification` that denotes a removal with the given [key] and [value].
  @literal const Modification.remove(this.key, this.value): kind = Kind.remove;

  /// Creates `Modification` that denotes the clearance of a collection without a key and value.
  @literal const Modification.clear(): kind = Kind.clear, key = null, value = null;

  @literal const Modification._(this.kind, this.key, this.value);

}