import 'package:meta/meta.dart';


/// Denotes a fake enumeration.
const enumeration = 'fake enumeration';


/// Denotes that the annotated value may be `null`.
const nullable = Nullable();

/// An annotation that denotes a value may be `null`.
@immutable class Nullable {

  /// The reason that the annotated value may be `null`.
  final String reason;

  /// Creates a `Nullable`.
  @literal const Nullable([this.reason = '']);

}


/// Denotes that the annotated value is unsanitized.
const tainted = Tainted();

/// An annotation that denotes an unsanitized value.
@immutable class Tainted {

  /// The source of the unsanitized value.
  final String source;

  /// Creates a `Tainted`.
  @literal const Tainted([this.source = '']);

}


/// Denotes that the annotated number is always positive.
const positive = NumericalConstraint();

/// Denotes that the annotated number is unsigned.
const unsigned = NumericalConstraint();

/// Denotes that the annotated number cannot be zero.
const nonZero = NumericalConstraint();

/// Denotes that the annotated number is negative.
const negative = NumericalConstraint();

/// An annotation that denotes a numerical constraint.
@immutable class NumericalConstraint {

  /// The reason which explains the constraint.
  final String reason;

  /// Creates a `NumericalConstraint`.
  @literal const NumericalConstraint([this.reason = '']);

}