import 'dart:async';

import 'package:meta/meta.dart';


/// A `Listenable` broadcasts a stream of events to which other types can listen.
mixin Listenable<T> {

  /// The source of events.
  @visibleForTesting
  @protected final StreamController<T> source = StreamController();


  /// The stream of events.
  Stream<T> get events => source.stream;


  /// Closes the stream of events.
  void dispose() => source.close();

}




