import 'package:meta/meta.dart';

import 'package:sane/core.dart';


/// A positive mathematical period. All fields must be greater than `0`.
@immutable class Period {

  /// An empty period.
  @enumeration static const Period none = Period();

  /// The number of years; must be greater than '0'.
  final int year;
  /// The number of months; must be greater than '0'.
  final int month;
  /// The number of days; must be greater than '0'.
  final int day;
  /// The number of hours; must be greater than '0'.
  final int hour;
  /// The number of minutes; must be greater than '0'.
  final int minute;
  /// The number of seconds; must be greater than '0'.
  final int second;
  /// The number of milliseconds; must be greater than '0'.
  final int millisecond;
  /// The number of microseconds; must be greater than '0'.
  final int microsecond;


  /// Creates a `Period` of the given duration.
  @literal const Period({
    this.year = 1,
    this.month = 1,
    this.day = 1,
    this.hour = 1,
    this.minute = 1,
    this.second = 1,
    this.millisecond = 1,
    this.microsecond = 1,
  }):
    assert(year > 0, 'Invalid year: $year, year must be positive'),
    assert(month > 0, 'Invalid month: $month, month must be positive'),
    assert(day > 0, 'Invalid day: $day, day must be positive'),
    assert(hour > 0, 'Invalid hour: $hour, hour must be positive'),
    assert(minute > 0, 'Invalid minute: $minute, minute must be positive'),
    assert(second > 0, 'Invalid second: $second, second must be positive'),
    assert (millisecond > 0, 'Invalid millisecond: $millisecond, millisecond must be positive'),
    assert(microsecond > 0, 'Invalid microsecond: $microsecond, microsecond must be positive');


  @override
  bool operator ==(Object other) => other is Period
                && year == other.year && month == other.month
                && day == other.day && hour == other.hour
                && minute == other.minute && second == other.second
                && millisecond == other.millisecond && microsecond == other.microsecond;

  @override
  int get hashCode => hash([year, month, day, hour, minute, second, millisecond, microsecond]);

  @override
  String toString() => 'Period{year: $year, month: $month, day: $day, hour: $hour, minute: $minute, second: $second, millisecond: $millisecond, microsecond: $microsecond}';

}