import 'package:sane/core.dart';

/// Converts the given `weekdays` into a bitfield with a length of 7. Each weekday
/// is assigned a value between 1 (Monday) to 7 (Sunday). A `1` at an index indicates
/// that the weekday assigned the value `7 - index`, is present.
///
/// For example, `{monday, wednesday, saturday, sunday}` will be packed as `1100101`.
@unsigned int pack(Iterable<int> weekdays) {
  assert(weekdays.every((val) => val >= 1 && val < 8), 'Invalid weekday in: $weekdays, weekday must be between 1 and 7');

  var value = 0;
  for (var weekday = 7; weekday >= 1; weekday--) {
    value <<= 1;
    if (weekdays.contains(weekday)) {
      value += 1;
    }
  }

  return value;
}

/// Converts the given `value` into a set of weekdays. The value is treated as a bitfield with a
/// length of 7. Each weekday is assigned a value between 1 (Monday) to 7 (Sunday). A `1` at an index indicates
/// that the weekday assigned the value `7 - index`, is present.
///
/// For example, the value `1100101` will be unpacked to `{monday, wednesday, saturday, sunday}`.
Set<int> unpack(@unsigned int value) {
  assert(value >= 0 && value < 128, 'Invalid value: $value, value must be between 0 and 127');

  final weekdays = <int>{};
  for (var weekday = 1; weekday <= 7; weekday++) {
    if (value.isOdd) {
      weekdays.add(weekday);
    }
    value >>= 1;
  }

  return weekdays;
}
