import 'package:test/test.dart';

import 'package:sane/collection.dart';


void main() {

  group('pair', () {

    const pair = Pair('a', 1);

    test('replace', () {
      final other = pair.replace(key: 'b', value: 2);

      expect(other.key, 'b');
      expect(other.value, 2);
    });

    test('map', () {
      final other = pair.map((key) => 2, (value) => 'b');

      expect(other.key, 2);
      expect(other.value, 'b');
    });

    test('reduce', () => expect(pair.reduce((key, value) => 'reduction'), 'reduction'));

    test('toList', () => expect(pair.toList(), ['a', 1]));

    group('==', () {

      // ignore: non_const_call_to_literal_constructor
      test('equals', () => expect(pair, Pair('a', 1)));

      test('incorrect type', () => expect(pair, isNot(const MapEntry('a', 1))));

      test('incorrect fields', () => expect(pair, isNot(const Pair('b', 2))));

    });

    group('hashCode', () {

      test('equals', () => expect(pair.hashCode, const Pair('a', 1).hashCode));

      test('not equals', () => expect(pair.hashCode, isNot(const MapEntry('a', 1).hashCode)));

    });

    test('toString', () => expect(pair.toString(), 'Pair{a: 1}'));

  });

  group('triple', () {

    const triple = Triple('a', 1, 'b');

    test('replace', () {
      final other = triple.replace(left: 'c', middle: 2, right: 'd');

      expect(other.left, 'c');
      expect(other.middle, 2);
      expect(other.right, 'd');
    });

    test('map', () {
      final other = triple.map((left) => 'c', (middle) => 2, (right) => 'd');

      expect(other.left, 'c');
      expect(other.middle, 2);
      expect(other.right, 'd');
    });

    test('reduce', () => expect(triple.reduce((left, middle, right) => 'reduction'), 'reduction'));

    test('toList', () => expect(triple.toList(), ['a', 1, 'b']));

    group('==', () {

      // ignore: non_const_call_to_literal_constructor
      test('equals', () => expect(triple, Triple('a', 1, 'b')));

      test('incorrect type', () => expect(triple, isNot(const MapEntry('a', 1))));

      test('incorrect fields', () => expect(triple, isNot(const Triple('a', 2, 'b'))));

    });

    group('hashCode', () {

      test('equals', () => expect(triple.hashCode, const Triple('a', 1, 'b').hashCode));

      test('not equals', () => expect(triple.hashCode, isNot(const Triple('a', 1, 'c').hashCode)));

    });

    test('toString', () => expect(triple.toString(), 'Triple{a, 1, b}'));

  });

  group('quad', () {

    const quad = Quad('a', 1, 'b', 2);

    test('replace', () {
      final other = quad.replace(first: 'c', second: 3, third: 'd', fourth: 4);

      expect(other.first, 'c');
      expect(other.second, 3);
      expect(other.third, 'd');
      expect(other.fourth, 4);
    });

    test('map', () {
      final other = quad.map((first) => 'c', (second) => 3, (third) => 'd', (fourth) => 4);

      expect(other.first, 'c');
      expect(other.second, 3);
      expect(other.third, 'd');
      expect(other.fourth, 4);
    });

    test('reduce', () => expect(quad.reduce((first, second, third, fourth) => 'reduction'), 'reduction'));

    test('toList', () => expect(quad.toList(), ['a', 1, 'b', 2]));

    group('==', () {

      // ignore: non_const_call_to_literal_constructor
      test('equals', () => expect(quad, Quad('a', 1, 'b', 2)));

      test('incorrect type', () => expect(quad, isNot(const MapEntry('a', 1))));

      test('incorrect fields', () => expect(quad, isNot(const Quad('a', 2, 'b', '2'))));

    });

    group('hashCode', () {

      test('equals', () => expect(quad.hashCode, const Quad('a', 1, 'b', 2).hashCode));

      test('not equals', () => expect(quad.hashCode, isNot(const Quad('a', 2, 'b', '2').hashCode)));

    });

    test('toString', () => expect(quad.toString(), 'Quad{a, 1, b, 2}'));

  });

}