import 'package:test/test.dart';

import 'package:sane/collection.dart';
import 'package:sane/core.dart';

import '../core/weak_test.dart';


class StubMap extends WeakMap<dynamic, dynamic> {

  bool cleaned = false;


  StubMap(Handle handle) : super(handle);

  @override
  void clean() => cleaned = true;

}

void main() {

  StubMap map;
  StubHandle handle;

  setUp(() => map = StubMap(handle = StubHandle()..mods = 5));


  group('[K] = V', () {

    test('handle contains key', () {
      handle.containment = true;
      map['a'] = 1;

      expect(map.delegate['a'], 1);
      expect(map.modifications, 4);
    });

    test('handle does not contain key', () {
      handle.containment = false;
      map['a'] = 1;

      expect(map.delegate['a'], isNull);
      expect(map.modifications, 5);
    });

  });

  group('remove', () {

    test('key exists', () {
      map.delegate['a'] = 1;

      expect(map.remove('a'), 1);
      expect(map.modifications, 4);
    });

    test('key not exists', () {
      expect(map.remove('a'), isNull);
      expect(map.modifications, 5);
    });

  });

  test('clear', () {
    expect((map..delegate['a'] = 1..clear()).modifications, 5);
    expect(map.delegate, isEmpty);
  });

  test('[K]', () {
    handle.containment = true;

    expect((map..['a'] = 42)['a'], 42);
    expect(map.cleaned, isTrue);
  });

  test('keys', () {
    handle.containment = true;

    expect((map..['a'] = 42).keys, isNotNull);
    expect(map.cleaned, isTrue);
  });

  test('cleanup', () {
    map..['a'] = 42..cleanup();

    expect(map.delegate, isEmpty);
  });

}

