import 'package:test/test.dart';

import 'package:sane/collection.dart';
import 'package:sane/core.dart';

import '../core/weak_test.dart';


class StubSet extends WeakSet<dynamic> {

  bool cleaned = false;

  StubSet(Handle handle) : super(handle);

  @override
  void clean() => cleaned = true;

}

void main() {

  StubSet set;
  StubHandle handle;

  setUp(() => set = StubSet(handle = StubHandle()..mods = 5));

  group('add', () {

    test('handle contains element', () {
      handle.containment = true;
      set.add(1);

      expect(set.set.contains(1), true);
      expect(set.modifications, 4);
    });

    test('handle does not contain element', () {
      handle.containment = false;
      set.add(1);

      expect(set.set.contains(1), false);
      expect(set.modifications, 5);
    });

  });

  group('remove', () {

    test('element exists', () {
      set.set.add(1);

      expect(set.remove(1), true);
      expect(set.modifications, 4);
    });

    test('element not exists', () {
      expect(set.remove(1), false);
      expect(set.modifications, 5);
    });

  });

  test('clear', () {
    expect((set..set.add(1)..clear()).modifications, 5);
    expect(set.set, isEmpty);
  });

  group('lookup', () {

    test('element does not exist', () => expect(set.lookup('a'), isNull));

    test('element exists', () {
      handle.containment = true;

      expect((set..set.add('a')).lookup('a'), 'a');
      expect(set.cleaned, true);
    });

  });

  test('iterator', () {
    expect(set.iterator, isNotNull);
    expect(set.cleaned, true);
  });

  test('length', () {
    expect(set.length, 0);
    expect(set.cleaned, true);
  });

  test('toSet', () {
    final other = set.toSet();

    expect(other, isNot(same(set)));
    expect(other.handle, same(set.handle));
    expect(other.set, same(set.set));
  });

  test('cleanup', () {
    set..add(42)..cleanup();

    expect(set.set, isEmpty);
  });

}
