import 'package:test/test.dart';

import 'package:sane/collection.dart';
import 'package:sane/core.dart';


void main() {

  group('FormattedNumber', () {

    test('padLeft', () => expect(1.padLeft(2), '01'));

    test('padRight', () => expect(1.padRight(2), '10'));

  });

  group('RangedNumber', () {

    group('between', () {

      test('min greater than max', () => expect(() => 1.between(3, 2), throwsA(predicate<AssertionError>((e) => e.message == 'Invalid bounds: 3 and 2, min cannot be greater than max'))));

      for (final values in [const Triple(1, 3, true), const Triple(2, 3, true), const Triple(2, 2, false), const Triple(3, 4, false)]) {
        test('is ${values.right ? '' : 'not'} in ${values.left} and ${values.middle}', () => expect(2.between(values.left, values.middle), values.right));
      }

    });

    group('outside', () {

      test('min greater than max', () => expect(() => 1.outside(3, 2), throwsA(predicate<AssertionError>((e) => e.message == 'Invalid bounds: 3 and 2, min cannot be greater than max'))));

      for (final values in [const Triple(3, 4, true), const Triple(0, 1, true), const Triple(2, 3, false), const Triple(1, 2, false)]) {
        test('is ${values.right ? '' : 'not'} outside ${values.left} and ${values.middle}', () => expect(2.outside(values.left, values.middle), values.right));
      }

    });

  });

  group('RoundableNumber', () {

    group('roundTo', () {

      for (final number in [3, 4, 5, 7]) {
        test('rounds $number', () => expect(number.roundTo(5), 5));
      }

      for (final number in [-3, 1, 2, 8]) {
        test('does not rounds $number', () => expect(number.roundTo(5), isNot(5)));
      }

      test('fails assertion', () => expect(() => 5.roundTo(0), throwsA(predicate<AssertionError>((e) => e.message == 'Number cannot be rounded about 0'))));

    });

    group('ceilTo', () {

      for (final number in [1, 2, 3, 5]) {
        test('ceils $number', () => expect(number.ceilTo(5), 5));
      }

      for (final number in [-3, -1, 0, 8]) {
        test('does not ceil $number', () => expect(number.ceilTo(5), isNot(5)));
      }

      test('fails assertion', () => expect(() => 5.ceilTo(0), throwsA(predicate<AssertionError>((e) => e.message == 'Number cannot be ceiled about 0'))));

    });

    group('floorTo', () {

      for (final number in [5, 6, 7, 9]) {
        test('floors $number', () => expect(number.floorTo(5), 5));
      }

      for (final number in [-3, 0, 3, 11]) {
        test('does not floor $number', () => expect(number.floorTo(5), isNot(5)));
      }

      test('fails assertion', () => expect(() => 5.floorTo(0), throwsA(predicate<AssertionError>((e) => e.message == 'Number cannot be floored about 0'))));

    });

  });

  test('hash', () => expect(hash([1508150, 1084855]), 56909678));

}