import 'package:test/test.dart';

import 'package:sane/core.dart';


class StubWeak extends Weak<dynamic> with Janitor<dynamic> {

  bool result = false;

  StubWeak(Handle<dynamic> handle) : super(handle);

  @override
  void cleanup() => result = true;

}

class StubJanitor extends Weak<dynamic> with Janitor<dynamic> {

  StubJanitor(Handle handle) : super(handle);
}

class StubHandle with Handle<dynamic> {

  bool containment = false;

  @override
  // ignore: type_annotate_public_apis
  bool contains(referent) => containment;

}


void main() {

  test('Weak modifications', () => expect(5, StubWeak(StubHandle()..mods = 5).modifications));

  group('Janitor', () {

    group('clean', () {

      test('different modifications', () {
        final weak = StubWeak(StubHandle()..mods = 5)..modifications = 3..clean();

        expect(weak.modifications, 5);
        expect(weak.result, true);
      });

      test('same modifications', () {
        final weak = StubWeak(StubHandle()..mods = 5)..modifications = 5..clean();

        expect(weak.modifications, 5);
        expect(weak.result, false);
      });

    });

    test('cleanup does nothing', StubJanitor(StubHandle()).cleanup);

  });

  group('WeakReference', () {

    group('exists', () {

      test('same modifications', () => expect(WeakReference(StubHandle(), 5).exists, true));

      test('different modification', () {
        final reference = WeakReference(StubHandle()..containment = true, 5)..modifications = 3;

        expect(reference.exists, true);
        expect(reference.modifications, 0);
        expect(reference.value, 5);
      });

      test('null value', () => expect(WeakReference(StubHandle()).exists, false));

      test('handle does not contain value', () {
        final reference = WeakReference(StubHandle(), 5)..modifications = 3;

        expect(reference.exists, false);
        expect(reference.modifications, 3);
        expect(reference.value, isNull);
      });

    });

    group('value', () {

      test('exists', () => expect(WeakReference(StubHandle()..containment = true, 5).value, 5));

      test('is null', () => expect(WeakReference(StubHandle()).value, isNull));

    });

  });

}