import 'package:test/test.dart';

import 'package:sane/time.dart';


const zero = 0;

void main() {

  group('assertions', () {

    test('year', () => expect(() => Period(year: zero), throwsA(predicate<AssertionError>((e) => e.message == 'Invalid year: 0, year must be positive'))));

    test('month', () => expect(() => Period(month: zero), throwsA(predicate<AssertionError>((e) => e.message == 'Invalid month: 0, month must be positive'))));

    test('day', () => expect(() => Period(day: zero), throwsA(predicate<AssertionError>((e) => e.message == 'Invalid day: 0, day must be positive'))));

    test('hour', () => expect(() => Period(hour: zero), throwsA(predicate<AssertionError>((e) => e.message == 'Invalid hour: 0, hour must be positive'))));

    test('minute', () => expect(() => Period(minute: zero), throwsA(predicate<AssertionError>((e) => e.message == 'Invalid minute: 0, minute must be positive'))));

    test('second', () => expect(() => Period(second: zero), throwsA(predicate<AssertionError>((e) => e.message == 'Invalid second: 0, second must be positive'))));

    test('millisecond', () => expect(() => Period(millisecond: zero), throwsA(predicate<AssertionError>((e) => e.message == 'Invalid millisecond: 0, millisecond must be positive'))));

    test('microsecond', () => expect(() => Period(microsecond: zero), throwsA(predicate<AssertionError>((e) => e.message == 'Invalid microsecond: 0, microsecond must be positive'))));

  });

  group('==', () {

    test('same', () => expect(Period.none, const Period()));

    test('different types', () => expect(Period.none, isNot(Duration.zero)));

    test('different values', () => expect(Period.none, isNot(const Period(second: 3))));

  });

  test('hashCode', () => expect(const Period(year: 1, month: 2, day: 3, hour: 4, minute: 5, second: 6, millisecond: 7, microsecond: 8).hashCode, 59812429836869));

  test('toString', () => expect(
    const Period(year: 1, month: 2, day: 3, hour: 4, minute: 5, second: 6, millisecond: 7, microsecond: 8).toString(),
    'Period{year: 1, month: 2, day: 3, hour: 4, minute: 5, second: 6, millisecond: 7, microsecond: 8}'
  ));

}