import 'package:test/test.dart';

import 'package:sane/time.dart';


void main() {

  group('DateTimeSpan', () {

    test('duration', () => expect(Span(DateTime.utc(0, 0, 3), DateTime.utc(0, 0, 5)).duration, const Duration(days: 2)));

  });

  group('TimeSpan', () {

    test('duration', () => expect(Span(Time.noon, Time.max).duration, const Duration(hours: 11, minutes: 59, seconds: 59, milliseconds: 999, microseconds: 999)));

  });

  group('NumericalSpan', () {

    test('duration', () => expect(Span(1, 5).duration, 4));

  });

  group('Span', () {

    test('overlaps', () {
      expect(Span(1, 3).overlaps(Span(1, 3)), true);
      expect(Span(1, 3).overlaps(Span(1, 2)), true);
      expect(Span(1, 3).overlaps(Span(2, 3)), true);

      expect(Span(1, 2).overlaps(Span(1, 1)), false);
      expect(Span(1, 2).overlaps(Span(2, 2)), false);

      expect(Span(1, 2).overlaps(Span(-1, 0)), false);
      expect(Span(1, 2).overlaps(Span(0, 1)), false);
      expect(Span(1, 2).overlaps(Span(2, 3)), false);
      expect(Span(1, 2).overlaps(Span(3, 4)), false);
    });

    group('compareTo', () {

      test('different weight', () => expect(Span(1, 3, 2).compareTo(Span(1, 3, 3)), -1));

      test('same', () => expect(Span(1, 2, 3).compareTo(Span(1, 2, 3)), 0));

    });

    group('==', () {

      test('same', () => expect(Span(1) == Span(1), true));

      // ignore: unrelated_type_equality_checks
      test('different type', () => expect(Span(1) == Span(Time.max), false));

    });

    test('hashCode', () => expect(Span(1, 2, 3).hashCode, 862547));

    test('toString', () => expect(Span(1, 2, 3).toString(), 'Span{weight: 3, duration: 1 to 2}'));

  });

}